@@ -101,7 +101,7 @@ dlg_prop (GtkWidget *widget,
 	label = glade_xml_get_widget (data->gui, "p_path_label");
 	/* window->archive_filename is unescaped. */
 	s = remove_level_from_path (window->archive_filename); 
-	utf8_name = g_locale_to_utf8 (s, -1, NULL, NULL, NULL);
+	utf8_name = g_filename_to_utf8 (s, -1, NULL, NULL, NULL);
 	gtk_label_set_text (GTK_LABEL (label), utf8_name);
 	g_free (utf8_name);
 	g_free (s);
@@ -113,7 +113,7 @@ dlg_prop (GtkWidget *widget,
 
 	label = glade_xml_get_widget (data->gui, "p_name_label");
 	s1 = file_name_from_path (window->archive_filename);
-	utf8_name = g_locale_to_utf8 (s1, -1, NULL, NULL, NULL);
+	utf8_name = g_filename_to_utf8 (s1, -1, NULL, NULL, NULL);
 	gtk_label_set_text (GTK_LABEL (label), utf8_name);
 
 	title_txt = g_strdup_printf (_("%s Properties"), utf8_name);
