@@ -24,20 +24,25 @@
 #include <string.h>
 #include <stdio.h>
 #include <glib.h>
+#include <gnome.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include "file-data.h"
 #include "file-list.h"
 #include "file-utils.h"
 #include "fr-archive.h"
 #include "fr-command.h"
+#include "fr-command-arj.h"
 #include "fr-command-cfile.h"
 #include "fr-command-lha.h"
 #include "fr-command-rar.h"
 #include "fr-command-tar.h"
 #include "fr-command-zip.h"
 #include "fr-command-zoo.h"
+#include "fr-error.h"
 #include "fr-marshal.h"
 #include "fr-process.h"
+#include "utf8-fnmatch.h"
+
 
 #define MAX_CHUNK_LEN 16000 /* FIXME : what is the max length of a command 
 			     * line ? */
@@ -46,6 +51,9 @@
 enum {
 	START,
 	DONE,
+	PROGRESS,
+	MESSAGE,
+	STOPPABLE,
 	LAST_SIGNAL
 };
 
@@ -111,10 +119,59 @@ fr_archive_class_init (FRArchiveClass *class)
 			      G_TYPE_NONE, 2,
 			      G_TYPE_INT,
 			      G_TYPE_POINTER);
+	fr_archive_signals[PROGRESS] =
+		g_signal_new ("progress",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (FRArchiveClass, progress),
+			      NULL, NULL,
+			      fr_marshal_VOID__DOUBLE,
+			      G_TYPE_NONE, 1,
+			      G_TYPE_DOUBLE);
+	fr_archive_signals[MESSAGE] =
+		g_signal_new ("message",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (FRArchiveClass, message),
+			      NULL, NULL,
+			      fr_marshal_VOID__STRING,
+			      G_TYPE_NONE, 1,
+			      G_TYPE_STRING);
+	fr_archive_signals[STOPPABLE] =
+                g_signal_new ("stoppable",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (FRArchiveClass, stoppable),
+			      NULL, NULL,
+			      fr_marshal_VOID__BOOL,
+			      G_TYPE_NONE, 
+			      1, G_TYPE_BOOLEAN);
 	
 	gobject_class->finalize = fr_archive_finalize;
 	class->start = NULL;
 	class->done = NULL;
+	class->progress = NULL;
+	class->message = NULL;
+}
+
+
+void
+fr_archive_stoppable (FRArchive *archive,
+		      gboolean   stoppable)
+{
+	g_signal_emit (G_OBJECT (archive), 
+		       fr_archive_signals[STOPPABLE],
+		       0,
+		       stoppable);
+}
+
+
+static gboolean
+archive_sticky_only_cb (FRProcess *process,
+			FRArchive *archive)
+{
+	fr_archive_stoppable (archive, FALSE);
+	return TRUE;
 }
 
 
@@ -123,9 +180,16 @@ fr_archive_init (FRArchive *archive)
 {
 	archive->filename = NULL;
 	archive->command = NULL;
-	archive->process = fr_process_new ();
 	archive->is_compressed_file = FALSE;
-	archive->fake_load = FALSE;
+
+	archive->fake_load_func = NULL;
+	archive->fake_load_data = NULL;
+
+	archive->process = fr_process_new ();
+	g_signal_connect (G_OBJECT (archive->process), 
+			  "sticky_only",
+			  G_CALLBACK (archive_sticky_only_cb),
+			  archive);
 }
 
 
@@ -181,7 +245,8 @@ create_command_from_mime_type (FRArchive  *archive,
 		archive->command = fr_command_tar_new (archive->process, 
 						       filename, 
 						       FR_COMPRESS_PROGRAM_BZIP2);
-	} else if (is_mime_type (mime_type, "application/zip")) {
+	} else if (is_mime_type (mime_type, "application/zip") ||
+		   is_mime_type (mime_type, "application/x-zip")) {
 		archive->command = fr_command_zip_new (archive->process, 
 						       filename);
 	} else if (is_mime_type (mime_type, "application/x-zoo")) {
@@ -190,6 +255,9 @@ create_command_from_mime_type (FRArchive  *archive,
 	} else if (is_mime_type (mime_type, "application/x-rar")) {
 		archive->command = fr_command_rar_new (archive->process, 
 						       filename);
+	} else if (is_mime_type (mime_type, "application/x-arj")) {
+		archive->command = fr_command_arj_new (archive->process, 
+						       filename);
 	} else 
 		return FALSE;
 
@@ -275,6 +343,8 @@ create_command_from_filename (FRArchive  *archive,
 			      const char *filename,
 			      gboolean    loading)
 {
+	archive->is_compressed_file = FALSE;
+
 	if (file_extension_is (filename, ".tar.gz")
 	    || file_extension_is (filename, ".tgz")) {
 		archive->command = fr_command_tar_new (archive->process, 
@@ -319,6 +389,9 @@ create_command_from_filename (FRArchive  *archive,
 	} else if (file_extension_is (filename, ".rar")) {
 		archive->command = fr_command_rar_new (archive->process, 
 						       filename);
+	} else if (file_extension_is (filename, ".arj")) {
+		archive->command = fr_command_arj_new (archive->process, 
+						       filename);
 	} else if (loading) {
 		if (file_extension_is (filename, ".gz")
 		    || file_extension_is (filename, ".z")
@@ -380,6 +453,9 @@ action_performed (FRCommand   *command,
 	case FR_ACTION_TEST:
 		s_action = "Test";
 		break;
+	case FR_ACTION_GET_LIST:
+		s_action = "Get list";
+		break;
 	}
 	g_print ("%s [DONE]\n", s_action);
 #endif
@@ -392,6 +468,32 @@ action_performed (FRCommand   *command,
 }
 
 
+static gboolean
+archive_progress_cb (FRCommand  *command,
+		     double      fraction,
+		     FRArchive  *archive)
+{
+	g_signal_emit (G_OBJECT (archive), 
+		       fr_archive_signals[PROGRESS], 
+		       0,
+		       fraction);
+	return TRUE;
+}
+
+
+static gboolean
+archive_message_cb  (FRCommand  *command,
+		     const char *msg,
+		     FRArchive  *archive)		     
+{
+	g_signal_emit (G_OBJECT (archive), 
+		       fr_archive_signals[MESSAGE], 
+		       0,
+		       msg);
+	return TRUE;
+}
+
+
 /* filename must not be escaped. */
 gboolean
 fr_archive_new_file (FRArchive  *archive, 
@@ -425,23 +527,57 @@ fr_archive_new_file (FRArchive  *archive,
 			  "done",
 			  G_CALLBACK (action_performed),
 			  archive);
+	g_signal_connect (G_OBJECT (archive->command), 
+			  "progress",
+			  G_CALLBACK (archive_progress_cb),
+			  archive);
+	g_signal_connect (G_OBJECT (archive->command), 
+			  "message",
+			  G_CALLBACK (archive_message_cb),
+			  archive);
 
 	return TRUE;
 }
 
 
+void
+fr_archive_set_fake_load_func (FRArchive    *archive,
+			       FakeLoadFunc  func,
+			       gpointer      data)
+{
+	archive->fake_load_func = func;
+	archive->fake_load_data = data;
+}
+
+
+gboolean
+fr_archive_fake_load (FRArchive *archive)
+{
+	if (archive->fake_load_func != NULL)
+		return (*archive->fake_load_func) (archive, archive->fake_load_data);
+	else
+		return FALSE;
+}
+
+
 /* filename must not be escaped. */
 gboolean
-fr_archive_load (FRArchive  *archive, 
-		 const char *filename)
+fr_archive_load (FRArchive   *archive, 
+		 const char  *filename,
+		 GError     **gerror)
 {
 	FRCommand  *tmp_command;
 	const char *mime_type;
 
 	g_return_val_if_fail (archive != NULL, FALSE);
 
-	if (access (filename, F_OK) != 0) /* file must exists. */
+	if (access (filename, F_OK) != 0) { /* file must exists. */
+		if (gerror != NULL) 
+			*gerror = g_error_new (fr_error_quark (),
+					       0,
+					       _("The file does not exist."));
 		return FALSE;
+	}
 
 	archive->read_only = access (filename, W_OK) != 0;
 
@@ -461,6 +597,11 @@ fr_archive_load (FRArchive  *archive,
 	    || ! create_command_from_mime_type (archive, filename, mime_type))
 		if (! create_command_from_filename (archive, filename, TRUE)) {
 			archive->command = tmp_command;
+			if (gerror != NULL) 
+				*gerror = g_error_new (fr_error_quark (),
+						       0,
+						       _("Archive type not supported."));
+
                         return FALSE;
 		}
 	
@@ -471,13 +612,21 @@ fr_archive_load (FRArchive  *archive,
 			  "start",
 			  G_CALLBACK (action_started),
 			  archive);
-
 	g_signal_connect (G_OBJECT (archive->command), 
 			  "done",
 			  G_CALLBACK (action_performed),
 			  archive);
+	g_signal_connect (G_OBJECT (archive->command), 
+			  "progress",
+			  G_CALLBACK (archive_progress_cb),
+			  archive);
+	g_signal_connect (G_OBJECT (archive->command), 
+			  "message",
+			  G_CALLBACK (archive_message_cb),
+			  archive);
 
-	archive->command->fake_load = archive->fake_load;
+	fr_archive_stoppable (archive, TRUE);
+	archive->command->fake_load = fr_archive_fake_load (archive);
 	fr_command_list (archive->command);
 
 	return TRUE;
@@ -490,7 +639,8 @@ fr_archive_reload (FRArchive *archive)
 	g_return_if_fail (archive != NULL);
 	g_return_if_fail (archive->filename != NULL);
 
-	archive->command->fake_load = archive->fake_load;
+	fr_archive_stoppable (archive, TRUE);
+	archive->command->fake_load = fr_archive_fake_load (archive);
 	fr_command_list (archive->command);
 }
 
@@ -506,7 +656,8 @@ fr_archive_rename (FRArchive  *archive,
 		/* If the archive is a compressed file we have to reload it,
 		 * because in this case the 'content' of the archive changes 
 		 * too. */
-		fr_archive_load (archive, filename);
+		fr_archive_load (archive, filename, NULL);
+
 	else {
 		if (archive->filename != NULL)
 			g_free (archive->filename);
@@ -529,7 +680,6 @@ find_file_in_archive (FRArchive *archive,
 
 	for (scan = archive->command->file_list; scan; scan = scan->next) {
 		FileData *fdata = scan->data;
-		
 		if (strcmp (path, fdata->original_path) == 0)
 			return fdata;
 	}
@@ -596,11 +746,11 @@ newer_files_only (FRArchive  *archive,
 
 /* Note: all paths unescaped. */
 void
-fr_archive_add (FRArchive   *archive, 
-		GList       *file_list, 
-		const char  *base_dir,
-		gboolean     update,
-		const char  *password,
+fr_archive_add (FRArchive     *archive, 
+		GList         *file_list, 
+		const char    *base_dir,
+		gboolean       update,
+		const char    *password,
 		FRCompression  compression)
 {
 	GList    *new_file_list;
@@ -614,6 +764,8 @@ fr_archive_add (FRArchive   *archive,
 	if (archive->read_only)
 		return;
 
+	fr_archive_stoppable (archive, FALSE);
+
 	/* if the command cannot update,  get the list of files that are 
 	 * newer than the ones in the archive. */
 
@@ -628,6 +780,12 @@ fr_archive_add (FRArchive   *archive,
 #ifdef DEBUG
 		g_print ("nothing to update.\n");
 #endif
+
+		archive->process->error.type = FR_PROC_ERROR_NONE;
+		g_signal_emit_by_name (G_OBJECT (archive->process), 
+				       "done",
+				       FR_ACTION_ADD,
+				       &archive->process->error);
 		return;
 	}
 
@@ -647,7 +805,7 @@ fr_archive_add (FRArchive   *archive,
 		GList *del_list = NULL;
 
 		for (scan = new_file_list; scan != NULL; scan = scan->next) {
-			gchar *filename = scan->data;
+			char *filename = scan->data;
 			if (find_file_in_archive (archive, filename)) 
 				del_list = g_list_prepend (del_list, filename);
 		}
@@ -663,6 +821,7 @@ fr_archive_add (FRArchive   *archive,
 	/* add now. */
 
 	e_file_list = escape_file_list (new_file_list);
+	fr_command_set_n_files (archive->command, g_list_length (e_file_list));
 
 	for (scan = e_file_list; scan != NULL; ) {
 		GList *prev = scan->prev;
@@ -695,7 +854,6 @@ fr_archive_add (FRArchive   *archive,
 		g_list_free (new_file_list);
 
 	fr_command_recompress (archive->command, compression);
-
 	fr_process_start (archive->process);
 }
 
@@ -714,14 +872,18 @@ file_list_remove_from_pattern (GList      **list,
 	
 	for (scan = *list; scan;) {
 		char *path = scan->data;
+		char *utf8_name;
 
-		if (match_patterns (patterns, file_name_from_path (path))) {
+		utf8_name = g_filename_to_utf8 (file_name_from_path (path), 
+						-1, NULL, NULL, NULL);
+		if (match_patterns (patterns, utf8_name, FNM_CASEFOLD)) {
 			*list = g_list_remove_link (*list, scan);
 			g_free (scan->data);
 			g_list_free (scan);
 			scan = *list;
 		} else
 			scan = scan->next;
+		g_free (utf8_name);
 	}
 	
 	g_strfreev (patterns);
@@ -790,6 +952,8 @@ fr_archive_add_with_wildcard (FRArchive     *archive,
 	if (archive->read_only)
 		return NULL;
 
+	fr_archive_stoppable (archive, TRUE);
+
 	aww_data = g_new0 (AddWithWildcardData, 1);
 	aww_data->archive = archive;
 	aww_data->base_dir = g_strdup (base_dir);
@@ -870,6 +1034,8 @@ fr_archive_add_directory (FRArchive     *archive,
 	if (archive->read_only)
 		return NULL;
 
+	fr_archive_stoppable (archive, TRUE);
+
 	ad_data = g_new0 (AddDirectoryData, 1);
 	ad_data->archive = archive;
 	ad_data->directory = g_strdup (directory);
@@ -904,6 +1070,7 @@ _archive_remove (FRArchive *archive,
 		file_list = archive->command->file_list;
 
 	e_file_list = escape_file_list (file_list);
+	fr_command_set_n_files (archive->command, g_list_length (e_file_list));
 
 	for (scan = e_file_list; scan != NULL; ) {
 		GList *prev = scan->prev;
@@ -941,6 +1108,8 @@ fr_archive_remove (FRArchive     *archive,
 	if (archive->read_only)
 		return;
 
+	fr_archive_stoppable (archive, FALSE);
+
 	fr_process_clear (archive->process);
 	fr_command_uncompress (archive->command);
 	_archive_remove (archive, file_list);
@@ -1036,6 +1205,8 @@ extract_in_chunks (FRCommand  *command,
 {
 	GList *scan;
 
+	fr_command_set_n_files (command, g_list_length (file_list));
+
 	if (file_list == NULL) {
 		fr_command_extract (command,
 				    file_list,
@@ -1095,6 +1266,8 @@ fr_archive_extract (FRArchive  *archive,
 
 	g_return_if_fail (archive != NULL);
 
+	fr_archive_stoppable (archive, TRUE);
+
 	/* if a command supports all the requested options use 
 	 * fr_command_extract directly. */
 
@@ -1255,7 +1428,10 @@ void
 fr_archive_test (FRArchive  *archive,
 		 const char *password)
 {
+	fr_archive_stoppable (archive, TRUE);
+
 	fr_process_clear (archive->process);
+	fr_command_set_n_files (archive->command, 0);
 	fr_command_test (archive->command, password);
 	fr_process_start (archive->process);
 }
@@ -1270,6 +1446,7 @@ G_CONST_RETURN char *
 fr_archive_utils__get_file_name_ext (const char *filename)
 {
 	static char * ext[] = {
+		".arj",
 		".bz", 
 		".bz2", 
 		".ear",
